﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Web;

namespace EPT_Tel_D_Emulator
{
    public partial class EPT_Tel_D_Emulator : Form
    {
        public EPT_Tel_D_Emulator()
        {
            InitializeComponent();

            for (int i = 0; i < EPTTransmitDevice.Length; ++i)
            {
                EPTTransmitDevice[i] = new Transfer();
            }
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            btnWriteByte.Enabled = false;

            for (int i = 0; i < 22; i++)
            {
                TelDcommand[i] = new t_TelDcommand();
            }
            InitTelDCommandArray();
        }


        //Index to store device selection
        Int32 device_index;

        //Create a Receive object of the Transfer Class.
        Transfer EPTReceiveData = new Transfer();

        //Create an array of the Transfer Class for device
        Transfer[] EPTTransmitDevice = new Transfer[8];

        //Tel-D object to store the current state of the Command Message
        Tel_D TelDReceiveData = new Tel_D();

        //Tel-D object store the current state of the Response Message
        Tel_D TelDTransmitData = new Tel_D();

        //Storage register for the length of the incoming
        //message from the Host
        public int CommandMessageLength=0;

        //Storage register for the length of the response
        //message to the Host
        public int ResponseMessageLength = 0;

        //Parameters
        public const byte TRIGGER_OUT_COMMAND = 0x1;
        public const byte TRANSFER_OUT_COMMAND = 0x2;
        public const byte BLOCK_OUT_COMMAND = 0x4;

        //Tel-D Command States for the Receive
        public const int TELD_STATE_IDLE       = 1;
        public const int TELD_STATE_CMD        = 2;
        public const int TELD_STATE_LEN        = 3;
        public const int TELD_STATE_MESSAGE    = 4;
        public const int TELD_STATE_CRC1       = 5;
        public const int TELD_STATE_CRC2       = 6;

        //Tel-D Response States for the Transmit
        public const int TELD_TRANSMITSTATE_IDLE        = 1;
        public const int TELD_TRANSMITSTATE_DUMMY       = 2;
        public const int TELD_TRANSMITSTATE_ACK         = 3;
        public const int TELD_TRANSMITSTATE_LEN         = 4;
        public const int TELD_TRANSMITSTATE_ACKREASON   = 5;
        public const int TELD_TRANSMITSTATE_STATUS      = 6;
        public const int TELD_TRANSMITSTATE_MESSAGE     = 7;
        public const int TELD_TRANSMITSTATE_CRC1        = 8;
        public const int TELD_TRANSMITSTATE_CRC2        = 9;


        //Tel-D Command Registesr
        public int Tel_DCommandCounter = 0;
        public byte [] Tel_DCommandBuf = new byte[10];
        public byte Tel_DResponseByte = 0;

        // Main object loader
        private void EPT_Tel_D_Emulator_Load(object sender, System.EventArgs e)
        {
            // Call the List Devices function
            ListDevices();

            // Set debug mode
            //EPT_AH_SetDebugMode(1);
        }

        private void SetButtonEnables()
        {
            btnOpenDevice.Enabled = false;
            btnCloseDevice.Enabled = true;
            btnWriteByte.Enabled = true;
        }

        private void btnOpenDevice_Click(object sender, EventArgs e)
        {
            //Open the Device
            OpenDevice();
            lblDeviceConnected.Text = "Device Connected";
        }

        private void btnCloseDevice_Click(object sender, EventArgs e)
        {
            if (EPT_AH_CloseDeviceByIndex(device_index) != 0)
            {
               btnWriteByte.Enabled = false;
               lblDeviceConnected.Text = " ";
			}
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            EPT_AH_CloseDeviceByIndex(device_index);
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            btnWriteByte.Enabled = false;

            lblDeviceConnected.Text = "";
            Application.Exit();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            EPT_AH_CloseDeviceByIndex(device_index);
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            btnWriteByte.Enabled = false;

            lblDeviceConnected.Text = "";
            Application.Exit();
        }

        private void btnWriteByte_Click(object sender, EventArgs e)
        {
            int i, address_to_device = 2;
            byte [] SendHexBuff;
            string result = "";

            //Counter to keep up with the number of bytes to send
            int SendHexBuffArrayNumOfBytes = 0;


            if (tbSPITransmit1.Text.Trim().Length != 0)
            {
                result += tbSPITransmit1.Text;
                SendHexBuffArrayNumOfBytes++;
            }

            if (tbSPITransmit2.Text.Trim().Length != 0)
            {
                result += tbSPITransmit2.Text;
                SendHexBuffArrayNumOfBytes++;
            }

            if (tbSPITransmit3.Text.Trim().Length != 0)
            {
                result += tbSPITransmit3.Text;
                SendHexBuffArrayNumOfBytes++;
            }

            if (tbSPITransmit4.Text.Trim().Length != 0)
            {
                result += tbSPITransmit4.Text;
                SendHexBuffArrayNumOfBytes++;
            }

            if (tbSPITransmit5.Text.Trim().Length != 0)
            {
                result += tbSPITransmit5.Text;
                SendHexBuffArrayNumOfBytes++;
            }


            string ModifiedResult = result.Replace(" ", "");
            string MetaResult = ModifiedResult.Replace("0x", "");
            string FinalResult = MetaResult.Replace(",", "");

            SendHexBuff = new byte[SendHexBuffArrayNumOfBytes];
            if ((SendHexBuff = StringtoByteArrayFastest(FinalResult)) == null)
                return;

            for (i = 0; i < SendHexBuffArrayNumOfBytes; i++)
            {
                EPT_AH_SendByte(address_to_device, (char)SendHexBuff[i]);
                EPT_AH_SendTrigger((byte)1);
                Thread.Sleep(10);
            }
        }

        public static byte[] StringtoByteArrayFastest(string hex)
        {
            if (hex.Length % 2 == 1)
            {
                MessageBox.Show("The Binary key cannot have an odd number of digits. Add a zero in front of single digits");
                return null;
            }

            byte[] arr = new byte[hex.Length >> 1];

            for (int i = 0; i < hex.Length >> 1; ++i)
            {
                arr[i] = (byte)((GetHexVal(hex[i << 1]) << 4) + (GetHexVal(hex[(i << 1) + 1])));
            }
            return arr;
        }

        public static int GetHexVal(char hex)
        {
            int val = (int)hex;
            //For uppercase A-F letters;
            return val - (val < 58 ? 48 : 55);
            //For lowercase a-f letters;
            //return val - (val < 58 ? 48 : 87);
            //Or the two combined, but a bit slower:
            //return val - (val < 58 ? 48 : (val <9 7? 55 : 87));
        }


        private void EPTParseReceive()
        {
            switch (EPTReceiveData.Command)
            {
                case TRIGGER_OUT_COMMAND:
                    TriggerOutReceive();
                    break;
                case TRANSFER_OUT_COMMAND:
                    TransferOutReceive();
                    break;
                default:
                    break;
            }
        }

        public void TransferOutReceive()
        {
            string WriteRcvChar = "";

            if (TelDReceiveData.TransferPending)
                return;

            WriteRcvChar = String.Format("{0:x2}", (int)System.Convert.ToUInt32(EPTReceiveData.cBlockBuf[0].ToString()));
            this.Invoke(new MethodInvoker(delegate() { tbDataBytes.AppendText(WriteRcvChar + " "); }));

            //WriteRcvChar = String.Format("{0}", (int)Tel_D_Command_Counter);
            //tbSPITransmit2.AppendText(WriteRcvChar + ", ");

            Tel_DCommandReceiveDecode(EPTReceiveData.cBlockBuf[0]);
        }

        public void Tel_DCommandReceiveDecode(byte Tel_DReceiveByte)
        {
            //
            // Tel-D Command State Machine
            /*
                TELD_STATE_IDLE         = 1
                TELD_STATE_CMD          = 2
                TELD_STATE_LEN          = 3
                TELD_STATE_MESSAGE      = 4
                TELD_STATE_CRC1         = 5
                TELD_STATE_CRC2         = 6
            */
            //
            //string WriteRcvChar = "";
            //int index, SendByteCount = 0;

            //WriteRcvChar = String.Format("{0}", (int)Tel_D_Command_Buf[0]);
            //tbSPITransmit3.AppendText(WriteRcvChar + ", ");

            //WriteRcvChar = String.Format("{0}", (int)Tel_D_Command_Buf[1]);
            //tbSPITransmit4.AppendText(WriteRcvChar + ", ");

            //WriteRcvChar = String.Format("{0}", (int)Tel_D_Command_Buf[2]);
            //tbSPITransmit5.AppendText(WriteRcvChar + ", ");

            switch (TelDReceiveData.CommandState)
            {
                case TELD_STATE_IDLE:
                    if (Tel_DReceiveByte < 0x16)
                    {
                        TelDReceiveData.Command = Tel_DReceiveByte;
                        TelDReceiveData.CommandState = TELD_STATE_CMD;
                    }
                    break;
                case TELD_STATE_CMD:
                    TelDReceiveData.Length = Tel_DReceiveByte;
                    CommandMessageLength = TelDReceiveData.Length;
                    TelDReceiveData.CommandState = TELD_STATE_LEN;
                    break;
                case TELD_STATE_LEN:
                    TelDReceiveData.Payload = new byte[TelDReceiveData.Length];
                    if(CommandMessageLength > 2)
                        TelDReceiveData.Payload[TelDReceiveData.Length - CommandMessageLength--] = Tel_DReceiveByte;
                    else
                        TelDReceiveData.CRC1 = Tel_DReceiveByte;
                        TelDReceiveData.CommandState = TELD_STATE_CRC1;
                    break;
                case TELD_STATE_MESSAGE:
                    TelDReceiveData.CommandState = TELD_STATE_CRC1;
                    break;
                case TELD_STATE_CRC1:
                    TelDReceiveData.CRC2 = Tel_DReceiveByte;
                    TelDReceiveData.TransferPending = true;
                    TelDReceiveData.CommandState = TELD_STATE_IDLE;
                    break;
                case TELD_STATE_CRC2:
                    TelDReceiveData.CommandState = TELD_STATE_IDLE;
                    TelDReceiveData.TransferPending = true;
                    break;
                default:
                    return;
                    break;
            }

        }

        public void TriggerOutReceive()
        {
            //switch ((int)EPTReceiveData.cBlockBuf[0])
            switch (EPTReceiveData.Payload)
            {
                case 0x01:
                    if(TelDReceiveData.TransferPending)
                        Tel_DResponseMessage();
                    break;
                default:
                    return;
                    break;
            }
        }

        public void Tel_DResponseMessage()
        {
            /*
             Tel-D Response State Machine
             Initiated by the Trigger EndTerm bit 0
             * 
             * 
             TELD_TRANSMITSTATE_IDLE        = 1;
             TELD_TRANSMITSTATE_DUMMY       = 2;
             TELD_TRANSMITSTATE_ACK         = 3;
             TELD_TRANSMITSTATE_LEN         = 4;
             TELD_TRANSMITSTATE_ACKREASON   = 5;
             TELD_TRANSMITSTATE_STATUS      = 6;
             TELD_TRANSMITSTATE_MESSAGE     = 7;
             TELD_TRANSMITSTATE_CRC1        = 8;
             TELD_TRANSMITSTATE_CRC2        = 9;
             * 
             */


            switch (TelDTransmitData.ResponseState)
            {
                case TELD_TRANSMITSTATE_IDLE:
                    Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byAckValue;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_DUMMY;
                    break;
                case TELD_TRANSMITSTATE_DUMMY:
                    Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byAckValue;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_ACK;
                    break;
                case TELD_TRANSMITSTATE_ACK:
                    Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byRxLength;
                    ResponseMessageLength = Tel_DResponseByte;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_LEN;
                    break;
                case TELD_TRANSMITSTATE_LEN:
                    Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byAckRsn;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_ACKREASON;
                    break;
                case TELD_TRANSMITSTATE_ACKREASON:
                    Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byStatus;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_STATUS;
                    break;
                case TELD_TRANSMITSTATE_STATUS:
                    if(ResponseMessageLength > 5)
                       TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_MESSAGE;
                    else
                        Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byCRC1;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_CRC1;
                    break;
                case TELD_TRANSMITSTATE_MESSAGE:
                    if (ResponseMessageLength > 0)
                        TelDReceiveData.Payload[TelDReceiveData.Length - ResponseMessageLength] = TelDcommand[TelDReceiveData.Command].Payload[TelDReceiveData.Length - ResponseMessageLength--];
                    else
                        Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byCRC1;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_CRC1;
                    break;
                case TELD_TRANSMITSTATE_CRC1:
                    Tel_DResponseByte = TelDcommand[TelDReceiveData.Command].byCRC2;
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_CRC2;
                    //TelDReceiveData.TransferPending = false;
                    break;
                case TELD_TRANSMITSTATE_CRC2:
                    TelDTransmitData.ResponseState = TELD_TRANSMITSTATE_IDLE;
                    TelDReceiveData.TransferPending = false;
                    break;
                default:
                    return;
                    break;
            }
                EPT_AH_SendByte(0x02, (char)Tel_DResponseByte);
                Thread.Sleep(1);
                EPT_AH_SendTrigger((byte)2);
                Thread.Sleep(1);
        }

        private void btnResetBlock_Click(object sender, EventArgs e)
        {
            tbDataBytes.Clear();
        }

        private void btnRstRcv_Click(object sender, EventArgs e)
        {
            tbDataBytes.Clear();
            tbSPITransmit2.Clear();
            tbSPITransmit3.Clear();
            tbSPITransmit4.Clear();
            tbSPITransmit5.Clear();
        }

        private void btnSPIReset_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte((char)2, (char)2);
            Thread.Sleep(1);
            EPT_AH_SendTransferControlByte((char)2, (char)0);
        }
    
    }

    public class Transfer
    {
        public int Command;
        public int Address;
        public int Length;
        public int Payload;
        public byte[] cBlockBuf;
        public bool TransferPending;
        public uint Repititions;

        public Transfer()
        {
            Command = 0;
            Address = 0;
            Length = 0;
            Payload = 0;
            TransferPending = false;
            Repititions = 0;
        }

    }

    public class Tel_D
    {
        public byte Command;
        public byte Dummy;
        public int Length;
        public byte[] Payload;
        public byte CRC1;
        public byte CRC2;
        public bool TransferPending;
        public int CommandState;
        public int ResponseState;

        public Tel_D()
        {
            Command = 0;
            Length = 0;
            CRC1 = 0;
            CRC2 = 0;
            TransferPending = false;
            CommandState = 1;
            ResponseState = 1;
        }

    }

}
