/*******************************************************************************
 * Filename:    ZigBprivate.h
 *
 * Created:     May 06, 2013
 *
 * Author:      Richard Jolly
 *
 * Revision History
 *         Date         Author  Comment
 *      2013_05_06      WCH     Initial Version
 *
*******************************************************************************/

#ifndef ZFM_ZigB_PRIVATE_H_
#define ZFM_ZigB_PRIVATE_H_


/*******************************************************************************
**
**      Defines
**
*******************************************************************************/
#define ZIGB_QUEUE_SIZE   4

#define ZIGB_TIMER_ACTIVATE( x )        { tx_timer_deactivate( &sZigBtimer ); tx_timer_change( &sZigBtimer, x, 0 ); tx_timer_activate( &sZigBtimer ); }
#define ZIGB_TIMER_DEACTIVATE()         { tx_timer_deactivate( &sZigBtimer ); }


/*******************************************************************************
**
**      Enums
**
*******************************************************************************/
typedef enum
{
  ZIGB_MESSAGE_INITIALIZE_TASK = ZFM_TASK_MESSAGE_BASE_ZIGB + 0x00000100,
  ZIGB_MESSAGE_EXTERNAL_REQUEST,
  ZIGB_MESSAGE_RESET_LINK,
  ZIGB_MESSAGE_INITIALIZE_LINK,
  ZIGB_MESSAGE_INTERRUPT_RECEIVED,
  ZIGB_MESSAGE_INTERRUPT_STATUS_FLAG,
  ZIGB_MESSAGE_SEND_COMMAND,
  
  //
  //    This message is not expected.
  //    Created to allow the psResponseQueue to have a message until configured
  //
  ZIGB_MESSAGE_RESPONSE,
  
  //
  //    Put all other messages above this line
  //
  ZIGB_MSG_MAX
  
} ZIGBtaskMessageEnum;


typedef enum
{
  ZIGB_STATE_BASE = ZFM_TASK_MESSAGE_BASE_ZIGB + 0x00000200,
  ZIGB_STATE_BEGIN,
  ZIGB_STATE_INTIALIZING_TASK,
  ZIGB_STATE_WAIT_CONFIGURATION,
  ZIGB_STATE_RESETING_LINK,
  ZIGB_STATE_INITIALIZING_LINK,
  ZIGB_STATE_LINK_INITIALIZED,
  ZIGB_STATE_SCANING,
  ZIGB_STATE_JOINING,
  ZIGB_STATE_JOINED,
  ZIGB_STATE_JOINED_OUT_OF_RANGE,
  ZIGB_STATE_REJOINING,
  
  //
  //    Put all other entries above this line
  //
  ZIGB_STATE_MAX
    
} ZIGBstateEnum;


typedef enum
{
  ZIGB_INTERRUPT_STATUS_FLAG_BASE = ZFM_TASK_MESSAGE_BASE_ZIGB + 0x00000300,
  ZIGB_INTERRUPT_STATUS_FLAG_ANOTHER_NETWORK,
  ZIGB_INTERRUPT_STATUS_FLAG_HIGH_RATE_TIMEOUT,
  ZIGB_INTERRUPT_STATUS_FLAG_RX_BUFFER_FULL,
  ZIGB_INTERRUPT_STATUS_FLAG_INVALID_PARAMETER,
  ZIGB_INTERRUPT_STATUS_FLAG_SCAN_JOIN_TIMEOUT,
  ZIGB_INTERRUPT_STATUS_FLAG_POWER_ON_RESET,
  ZIGB_INTERRUPT_STATUS_FLAG_RECEIVE_DATA_AVAILABLE,
  ZIGB_INTERRUPT_STATUS_FLAG_NETWORK_CONFIGURATION_CHANGE,
  ZIGB_INTERRUPT_STATUS_CLEAR_TX,
  
  //
  //    Put all other entries above this line
  //
  ZIGB_INTERRUPT_STATUS_FLAG_MAX
  
} ZIGBstatusFlagEnum;

typedef enum
{
  ZIGB_SEND_COMMAND_BASE = ZFM_TASK_MESSAGE_BASE_ZIGB + 0x00000400,
  ZIGB_SEND_COMMAND_INITIALIZE,
  ZIGB_SEND_COMMAND_JOIN,
  
  //
  //    Put all other entries above this line
  //
  ZIGB_SEND_COMMAND_MAX
  
} ZIGBsendCommandEnum;

typedef enum
{
  ZIGB_COMMAND_INITIALIZE,
  ZIGB_COMMAND_SCAN_NETWORK,
  ZIGB_COMMAND_JOIN_NETWORK,
  ZIGB_COMMAND_LEAVE_NETWORK,
  ZIGB_COMMAND_TRANSMIT_PACKET,
  ZIGB_COMMAND_READ_DATA,
  ZIGB_COMMAND_READ_STATUS,
  ZIGB_COMMAND_READ_NETWORK_STATUS,
  ZIGB_COMMAND_READ_REVISION,
  ZIGB_COMMAND_SET_INTERRUPT_FLAGS,
  ZIGB_COMMAND_NETWORK_MAC_IDS,
  ZIGB_COMMAND_SET_SECURITY_MODE,
  ZIGB_COMMAND_READ_STATISTICS,
  ZIGB_COMMAND_SET_RF_MODE,
  ZIGB_COMMAND_SET_RF_PARAMETERS,
  ZIGB_COMMAND_CLEAR_STATUS,
  ZIGB_COMMAND_PURGE_PACKET,
  ZIGB_COMMAND_READ_RF_PARAMETERS,
  ZIGB_COMMAND_WRITE_UPGRADE_PACKET,
  ZIGB_COMMAND_SET_LINK_KEY,
  ZIGB_COMMAND_SET_CCA_THRESHOLD,
  
  //
  //    Maximum ZIGB Command value
  //
  ZIGB_COMMAND_MAX
  
} ZIGBcommandEnum;


/*******************************************************************************
**
**      Structs
**
*******************************************************************************/
typedef struct t_ZIGBcommand
{
  ZIGBcommandEnum       eZIGBcommand;
  ZFM_U08                byCommandValue;
  ZFM_U08                byCRCindex;
  ZFM_U08                byTxRxLength;
  
} ZIGBcommand;

typedef struct t_NetworkTableEntry
{
  ZFM_U08        abyMACid[8];
  ZFM_U08        byStatus;
  ZFM_U08        byRFchannel;
  ZFM_U08        bySyncInterval;
  ZFM_U08        byBeaconInterval;
  ZFM_U08        byLQI;
  ZFM_U08        byMissedBeaconIntervals;
  
  ZFM_FLAG       fValid;
  ZFM_FLAG       fActiveInNetwork;
  
} NetworkTableEntry;

typedef struct t_RxDataPacket
{
  ZFM_FLAG       fInUse;
  ZFM_U08        byIndex;
  ZFM_U08        abyData[256];
  ZFM_U08        byRxPacketID;
  ZFM_U08        byLQI;
  ZFM_U08        byResponseType;
  ZFM_U08        byLength;
  ZFM_U08        abySourceMAC[8];
  ZFM_U08        abyDestinationMAC[8];

} RxDataPacket;

typedef struct t_TxDataPacket
{
  ZFM_FLAG       fInUse;
  ZFM_U08        byIndex;
  ZFM_U08       *pbyData;
  ZFM_U08        byLength;
  ZFM_U08        byTxPacketID;
  ZFM_U08        byTxFlags;
  ZFM_U32        uiTxHandle;

} TxDataPacket;


typedef struct t_ZIGBinfo
{
  ZIGBstateEnum                 eZIGBstate;
  
  ZFM_U08                        abyIndexToChannel[5];
  
  NetworkTableEntry             asNetworkEntries[4];
  ZFM_U08                        byThisNodeIndex;
  ZFM_U08                        byCoordinatorNodeIndex;
  NetworkCoordinatorTypeEnum    eConnectionType;
  ZFM_U16                        usStatusFlags;
  ZFM_U16                        usInterruptFlags;
  ZFM_U08                        byCurrentRFchannel;
  ZFM_U08                        byTxDisabledState;
  ZFM_U08                        byHighRateTimeout;
  ZFM_U08                        byMaxChannelScans;
  ZFM_U08                        byMaxMissedBeacons;
  ZFM_U08                        byMaxTxRetries;

  ZFM_U08                        byAckNak;
  ZFM_FLAG                       fHighRateMode;
  ZFM_FLAG                       fReceivedDataReady;

  ZFM_U16                        uiFirmwareRevision;
  ZFM_U16                        uiRFprotocolRevision;
  ZFM_U08                        abyRevisionMAC[8];
      
  RxDataPacket                  sRxData[4];
  TxDataPacket                  sTxData[4];
  TxDataPacket                 *psTxData;
  ZFM_U08                        byTxSequence;

  //
  //    Response configuration elements
  //
  ZFM_U32                uiResponseMessageID;
  
  //
  //    Serial Number and MAC address configuration elements
  //
  ZFM_U08                abySerialNumberZFMB[10];
  ZFM_U08                abyMACaddressZFMB[8];
  ZFM_U08                abySerialNumberPSC[10];
  ZFM_U08                abyMACaddressPSC[8];
  ZFM_U08                abySerialNumberBGP[10];
  ZFM_U08                abyMACaddressBGP[8];

  //
  //    Buffer used to build, send and receive commands
  //
  ZFM_U08                abyCommandBuffer[256];
  
} ZigBinfo;


//
//      ZIGB Statistics
//
typedef struct t_ZIGBstats
{
  ZFM_U32        uiTxWaitAbortedCount;
  ZFM_U32        uiCRCmismatch;
  ZFM_U32        uiTimerExpire;
  ZFM_U32        uiInterruptsReceived;
  ZFM_U32        uiInterruptMessagesProcessed;
  ZFM_U32        uiAnotherNetwork;
  ZFM_U32        uiHighRateTimeout;
  ZFM_U32        uiRxBufferFull;
  ZFM_U32        uiInvalidParameter;
  ZFM_U32        uiScanOrJoinTimeout;
  ZFM_U32        uiPowerOnReset;
  ZFM_U32        uiReceivedPacketAvailable;
  ZFM_U32        uiNetworkConfigurationChange;
  ZFM_U32        uiTxPacketReject;
  ZFM_U32        uiTxPacketFailed;
  ZFM_U32        uiTxPacketSucceeded;

  //
  //    Timer Expiring counters
  //
  ZFM_U32        uiTimerExpireStateBegin;
  ZFM_U32        uiTimerExpireInitializingTask;
  ZFM_U32        uiTimerExpireWaitConfiguration;
  ZFM_U32        uiTimerExpireResetingLink;
  ZFM_U32        uiTimerExpireInitializingLink;
  ZFM_U32        uiTimerExpireLinkInitialized;
  ZFM_U32        uiTimerExpireScaning;
  ZFM_U32        uiTimerExpireJoining;
  ZFM_U32        uiTimerExpireJoined;
  ZFM_U32        uiTimerExpireJoinedOutOfRange;
  ZFM_U32        uiTimerExpireRejoining;

  //
  //    Network Request Config counters
  //  
  ZFM_U32        uiNetworkRequestConfigSetResponse;
  ZFM_U32        uiNetworkRequestConfigSetSerialNumberZFMB;
  ZFM_U32        uiNetworkRequestConfigSetMACaddressZFMB;
  ZFM_U32        uiNetworkRequestConfigSetSerialNumberPSC;
  ZFM_U32        uiNetworkRequestConfigSetMACaddressPSC;
  ZFM_U32        uiNetworkRequestConfigSetSerialNumberBGP;
  ZFM_U32        uiNetworkRequestConfigSetMACaddressBGP;
  ZFM_U32        uiNetworkRequestConfigDefault;

  //
  //    Network Request Control counters
  //
  ZFM_U32        uiNetworkRequestControlStart;
  ZFM_U32        uiNetworkRequestControlConnect;
  ZFM_U32        uiNetworkRequestControlConnectStartScan;
  ZFM_U32        uiNetworkRequestControlDisconnect;
  ZFM_U32        uiNetworkRequestControlDefault;

  //
  //    Network Request Data counters
  //
  ZFM_U32        uiNetworkRequestDataTxData;
  ZFM_U32        uiNetworkRequestDataTxDataReject;
  ZFM_U32        uiNetworkRequestDataTxDataRejectNoBuffers;
  ZFM_U32        uiNetworkRequestDataTxDataRejectNotConnected;
  ZFM_U32        uiNetworkRequestDataRxDataAck;
  ZFM_U32        uiNetworkRequestDataDefault;
  
} ZigBstats;

extern ZigBstats        sZigBstats;


#endif /* ZFM_ZIGB_PRIVATE_H_ */
